package com.zjinja.mcmod.decor.elements.blocks;

import com.zjinja.mcmod.decor.ZJinJaDecoration;
import com.zjinja.mcmod.decor.element_base.EBaseTileBlock;
import com.zjinja.mcmod.decor.element_base.EBaseTileBlockForResourceSpecifiedRenderer;
import com.zjinja.mcmod.decor.element_templates.renderer.RTE_ExitSignWithSensor;
import com.zjinja.mcmod.decor.element_templates.renderer.TileRenderResourceSpecifiedRenderer;
import com.zjinja.mcmod.decor.elements.te.TE_ExitSignWithSensor;
import com.zjinja.mcmod.decor.elements.te.TE_RenderResourceSpecifiedTileEntity;
import com.zjinja.mcmod.decor.misc.ConfigLoader;
import com.zjinja.mcmod.decor.misc.RenderResource;
import com.zjinja.mcmod.decor.utils.model_dir_util.BoundingBoxVector;
import com.zjinja.mcmod.decor.utils.model_dir_util.OffsetVector;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.ChatComponentText;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import org.apache.logging.log4j.Level;

import java.util.List;
import java.util.Random;

public class B_ExitSignWithSensor extends EBaseTileBlock {
    public RenderResource NormalRender;
    public RenderResource DetectedRender;
    public RenderResource EmergencyRender;

    public B_ExitSignWithSensor() {
        super(Material.wood, TE_ExitSignWithSensor.class);
        setBasicProperty(2.0F, 0.1F, 10.0F, Block.soundTypeWood);
        DirUtil.FastDefine_4Dir();
        NormalRender = new RenderResource();
        DetectedRender = new RenderResource();
        EmergencyRender = new RenderResource();
        defineBlockInfo();
    }

    protected void defineBlockInfo(){
        DirUtil.DefineRelBoundingBox(new BoundingBoxVector(0.0F, 0.25F, 0.0F, 1.0F, 0.6875F, 0.0625F));
        NormalRender.LoadBlockResource("h7fsign", "exitsignwsen_n");
        DetectedRender.LoadBlockResource("h7fsign", "exitsignwsen_s");
        EmergencyRender.LoadBlockResource("h7fsign", "exitsignwsen_e");
    }

    @Override
    public void onBlockPlacedBy(World world, int x, int y, int z, EntityLivingBase player, ItemStack itemStack){
        DirUtil.FastPlaceConv_4Dir(world, x, y, z, player, itemStack);
    }

    @Override
    public int onBlockPlaced(World world, int x, int y, int z, int side, float hitX, float hitY, float hitZ, int smeta)
    {
        return side;
    }

    @Override
    public void onBlockAdded(World world, int x, int y, int z){
        super.onBlockAdded(world, x, y, z);
        world.scheduleBlockUpdate(x, y, z, this, 5);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public Object GetTileRenderer(){
        return new RTE_ExitSignWithSensor();
    }

    @Override
    public TileEntity createNewTileEntity(World world, int meta) {
        return new TE_ExitSignWithSensor();
    }


    @Override
    public boolean onBlockActivated(World world, int x, int y, int z, EntityPlayer player, int side, float hitX, float hitY, float hitZ)
    {
        if(!world.isRemote) {
            TileEntity ent = world.getTileEntity(x, y, z);
            if(!(ent instanceof  TE_ExitSignWithSensor)){
                return false;
            }
            TE_ExitSignWithSensor entity = (TE_ExitSignWithSensor)ent;
            entity.isEmergency = !entity.isEmergency;
            if (entity.isEmergency) {
                player.addChatMessage(new ChatComponentText(
                        ZJinJaDecoration.I18N.translateKey("cmsg.exitsignwsen.emgena")));
            } else {
                player.addChatMessage(new ChatComponentText(
                        ZJinJaDecoration.I18N.translateKey("cmsg.exitsignwsen.emgdis")));
            }
            world.markBlockForUpdate(entity.xCoord, entity.yCoord, entity.zCoord);
            entity.markDirty();
            NotifyPower(world, x, y, z);
        }
        return true;
    }

    public void NotifyPower(World world, int x, int y, int z){
        int meta = world.getBlockMetadata(x, y, z);
        Block thisblock = world.getBlock(x,y,z);
        OffsetVector ov1 = DirUtil.CalcOffsetFromRelOffset(meta, 0,0,-1);
        OffsetVector ov2 = DirUtil.CalcOffsetFromRelOffset(meta, 0,-1,-1);
        OffsetVector ov3 = DirUtil.CalcOffsetFromRelOffset(meta, 0,-2,-1);
        world.notifyBlockOfNeighborChange(ov1.getIntX() + x, ov1.getIntY() + y, ov1.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov2.getIntX() + x, ov2.getIntY() + y, ov2.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov3.getIntX() + x, ov3.getIntY() + y, ov3.getIntZ() + z, thisblock);
    }

    @Override
    public boolean canProvidePower()
    {
        return true;
    }

    @Override
    public boolean canConnectRedstone(IBlockAccess world, int x, int y, int z, int side){return true;}

    @Override
    public int isProvidingStrongPower(IBlockAccess ib, int x, int y, int z, int side)
    {
        TileEntity ent = ib.getTileEntity(x, y, z);
        if(!(ent instanceof  TE_ExitSignWithSensor)){
            return 0;
        }
        TE_ExitSignWithSensor entity = (TE_ExitSignWithSensor)ent;
        return (entity.isEmergency || entity.isDetected) ? 15 : 0;
    }

    @Override
    public int isProvidingWeakPower(IBlockAccess ib, int x, int y, int z, int side)
    {
        return isProvidingStrongPower(ib, x, y, z, side);
    }

    @Override
    public int tickRate(World world){
        return 5;
    }

    protected AxisAlignedBB getDetectionBox(World world, int x, int y, int z){
        float DetDis = (float) ConfigLoader.Conf.DoorSensorDetectDistance;
        int DetH = ConfigLoader.Conf.DoorSensorDetectHeight;
        int meta = world.getBlockMetadata(x, y, z);
        OffsetVector ov1 = DirUtil.CalcOffsetFromRelOffset(meta, -0.5F, -DetH, DetDis - 1.0F);
        OffsetVector ov2 = DirUtil.CalcOffsetFromRelOffset(meta, 0.5F, 0, -DetDis - 1.0F);
        float x1,y1,z1,x2,y2,z2;
        x1 = Math.min(ov1.X, ov2.X) + x;
        x2 = Math.max(ov1.X, ov2.X) + x;
        y1 = Math.min(ov1.Y, ov2.Y) + y;
        y2 = Math.max(ov1.Y, ov2.Y) + y;
        z1 = Math.min(ov1.Z, ov2.Z) + z;
        z2 = Math.max(ov1.Z, ov2.Z) + z;
        return AxisAlignedBB.getBoundingBox(x1, y1, z1, x2, y2, z2);
    }

    @Override
    public void updateTick(World world, int x, int y, int z, Random random) {
        if(!world.isRemote) {
            world.scheduleBlockUpdate(x, y, z, world.getBlock(x, y, z), 5);
            TileEntity ent = world.getTileEntity(x, y, z);
            if(!(ent instanceof  TE_ExitSignWithSensor)){
                return;
            }
            TE_ExitSignWithSensor entity = (TE_ExitSignWithSensor)ent;
            AxisAlignedBB aabb = getDetectionBox(world, x, y, z);
            List list = world.getEntitiesWithinAABB(EntityPlayer.class, aabb);
            if ((list != null) && (!list.isEmpty())) {
                if (!entity.isDetected) {
                    entity.isDetected = true;
                    world.markBlockForUpdate(entity.xCoord, entity.yCoord, entity.zCoord);
                    entity.markDirty();
                    NotifyPower(world, x, y, z);
                }
            } else {
                if (entity.isDetected) {
                    entity.isDetected = false;
                    world.markBlockForUpdate(entity.xCoord, entity.yCoord, entity.zCoord);
                    entity.markDirty();
                    NotifyPower(world, x, y, z);
                }
            }
        }
    }
}
