package com.zjinja.mcmod.decor.elements.blocks;

import com.zjinja.mcmod.decor.ZJinJaDecoration;
import com.zjinja.mcmod.decor.misc.ConfigLoader;
import com.zjinja.mcmod.decor.utils.model_dir_util.BoundingBoxVector;
import com.zjinja.mcmod.decor.utils.model_dir_util.OffsetVector;
import com.zjinja.mcmod.decor.utils.model_dir_util.PlaceSideEnum;
import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.World;

public class B_ExitSignWithSensorTDM extends B_ExitSignWithSensor {

    @Override
    protected void defineBlockInfo(){
        DirUtil.DefineRelBoundingBox(new BoundingBoxVector(0.5F, 0.25F, 0.0F, 1.0F, 0.6875F, 0.0625F));
        NormalRender.LoadBlockResource("h7ftdsign", "exitsignwsen_n");
        DetectedRender.LoadBlockResource("h7ftdsign", "exitsignwsen_s");
        EmergencyRender.LoadBlockResource("h7ftdsign", "exitsignwsen_e");
    }

    @Override
    public boolean canPlaceBlockOnSide(World world, int x, int y, int z, int side){
        if(side < 2) return false;
        int meta = PlaceSideEnum.TransToRotation(PlaceSideEnum.valueOf(side));
        OffsetVector ov = DirUtil.CalcOffsetFromRelOffset(meta, 1,0,0);
        int nsx = x + ov.getIntX();
        int nsy = y + ov.getIntY();
        int nsz = z + ov.getIntZ();
        return world.isAirBlock(nsx, nsy, nsz);
    }

    @Override
    public void onBlockPlacedBy(World world, int x, int y, int z, EntityLivingBase player, ItemStack itemStack){
        DirUtil.FastPlaceConv_4Dir(world, x, y, z, player, itemStack);
        int meta = world.getBlockMetadata(x, y, z);
        OffsetVector ov = DirUtil.CalcOffsetFromRelOffset(meta, 1,0,0);
        int nsx = x + ov.getIntX();
        int nsy = y + ov.getIntY();
        int nsz = z + ov.getIntZ();
        Block SlaveBlock = ZJinJaDecoration.proxy.Registry.GetBlockByName("exitsignwsentds");
        world.setBlock(nsx, nsy, nsz, SlaveBlock, meta, 2);
    }

    @Override
    public void breakBlock(World world, int x, int y, int z, Block block, int meta){
        OffsetVector ov = DirUtil.CalcOffsetFromRelOffset(meta, 1,0,0);
        int nsx = x + ov.getIntX();
        int nsy = y + ov.getIntY();
        int nsz = z + ov.getIntZ();
        super.breakBlock(world, x, y, z,block, meta);
        world.setBlockToAir(nsx, nsy, nsz);
    }

    @Override
    protected AxisAlignedBB getDetectionBox(World world, int x, int y, int z){
        float DetDis = (float) ConfigLoader.Conf.DoorSensorDetectDistance;
        int DetH = ConfigLoader.Conf.DoorSensorDetectHeight;
        int meta = world.getBlockMetadata(x, y, z);
        OffsetVector ov1 = DirUtil.CalcOffsetFromRelOffset(meta, -0.5F, -DetH, DetDis - 1.0F);
        OffsetVector ov2 = DirUtil.CalcOffsetFromRelOffset(meta, 1.5F, 0, -DetDis - 1.0F);
        float x1,y1,z1,x2,y2,z2;
        x1 = Math.min(ov1.X, ov2.X) + x;
        x2 = Math.max(ov1.X, ov2.X) + x;
        y1 = Math.min(ov1.Y, ov2.Y) + y;
        y2 = Math.max(ov1.Y, ov2.Y) + y;
        z1 = Math.min(ov1.Z, ov2.Z) + z;
        z2 = Math.max(ov1.Z, ov2.Z) + z;
        return AxisAlignedBB.getBoundingBox(x1, y1, z1, x2, y2, z2);
    }

    @Override
    public void NotifyPower(World world, int x, int y, int z){
        int meta = world.getBlockMetadata(x, y, z);
        Block thisblock = world.getBlock(x,y,z);
        OffsetVector ov1 = DirUtil.CalcOffsetFromRelOffset(meta, 0,0,-1);
        OffsetVector ov2 = DirUtil.CalcOffsetFromRelOffset(meta, 0,-1,-1);
        OffsetVector ov3 = DirUtil.CalcOffsetFromRelOffset(meta, 0,-2,-1);
        OffsetVector ov4 = DirUtil.CalcOffsetFromRelOffset(meta, 1,0,-1);
        OffsetVector ov5 = DirUtil.CalcOffsetFromRelOffset(meta, 1,-1,-1);
        OffsetVector ov6 = DirUtil.CalcOffsetFromRelOffset(meta, 1,-2,-1);
        world.notifyBlockOfNeighborChange(ov1.getIntX() + x, ov1.getIntY() + y, ov1.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov2.getIntX() + x, ov2.getIntY() + y, ov2.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov3.getIntX() + x, ov3.getIntY() + y, ov3.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov4.getIntX() + x, ov4.getIntY() + y, ov4.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov5.getIntX() + x, ov5.getIntY() + y, ov5.getIntZ() + z, thisblock);
        world.notifyBlockOfNeighborChange(ov6.getIntX() + x, ov6.getIntY() + y, ov6.getIntZ() + z, thisblock);
    }

}
